/*
*********************************************************************************
*                            In the name of Almighty                            *
*                                                                               *
*          SoccerTypes.h : Robocup 3D Soccer Simulation Team Zigorat            *
*                     (This team was previously named Gcyrus)                   *
*                                                                               *
*  Date: 03/20/2007                                                             *
*  Author: Mahdi Hamdarsi                                                       *
*  Comments: This file contains class decleration for SoccerCommand             *
*            (Some materials are owned & copyrighted by Jelle Kok)              *
*                                                                               *
*********************************************************************************
*/

/*! \file SoccerTypes.h
<pre>
<b>File:</b>          SoccerTypes.h
<b>Project:</b>       Robocup Soccer Simulation Team: Zigorat
<b>Authors:</b>       Mahdi Hamdarsi
<b>Created:</b>       03/20/2007
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains class decleration for SoccerCommand
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
03/20/2007       Mahdi           Initial version created
</pre>
*/


/*
Copyright (c) 2000-2003, Jelle Kok, University of Amsterdam
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Amsterdam nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*!
<pre>
<b>File:</b>          SoccerTypes.h
<b>Project:</b>       Robocup Soccer Simulation Team: UvA Trilearn
<b>Authors:</b>       Jelle Kok
<b>Created:</b>       28/11/2000
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains the different enumerations and
               constants that are important in the Soccer Server.
               Furthermore it contains the class SoccerCommand which is
               used to denote the different possible soccer commands and
               the class SoccerTypes that contains all kind of static
               methods to translate text strings that are received from
               the server into the soccer types (=enumerations) that
               are defined here. Finally it contains the Time class which
               holds a two-tuple that represents the time in the soccer server.
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
28/11/2000       Jelle Kok       Initial version created
</pre>
*/

#ifndef _SOCCERTYPES_
#define _SOCCERTYPES_

#include "Geometry.h"     // needed for AngDeg


/*****************************************************************************/
/********************* DEFINES ***********************************************/
/*****************************************************************************/

#define MAX_TEAMMATES          11       /*!< Maximum number of teammates     */
#define MAX_OPPONENTS          11       /*!< Maximum number of opponents     */
#define MAX_MSG              4096       /*!< maximum message size from server*/
#define MAX_SAY_MSG            10       /*!< maximum size of say message     */
#define MAX_FLAGS               8       /*!< maximum number of flags on field*/
#define MIN_FIELD_LENGTH       10       /*!< minimum length field can have   */
#define MIN_FIELD_WIDTH        10       /*!< minimum width field can have    */


/*! The time in Soccer Simulation 3D is a float with cycles of 0.02 second */
typedef float Time;

/*****************************************************************************/
/********************* CONSTANTS *********************************************/
/*****************************************************************************/


const double  UnknownDoubleValue  = -1000.0; /*!< indicates unknown double   */
const AngDeg  UnknownAngleValue   = -1000.0; /*!< indicates unknown angle    */
const Time    UnknownTime         = -1000.0;     /*!< indicates unknown time     */
const int     UnknownIntValue     = -1000;   /*!< indicates unknown int      */


/*****************************************************************************/
/********************** ENUMERATIONS *****************************************/
/*****************************************************************************/

/*! ObjectT is an enumeration of all possible objects that are part of the
    RoboCup soccer simulation. The class SoccerTypes contains different methods
    to easily work with these objects and convert them to text strings and text
    strings to ObjectT. */
enum ObjectT { // don't change order
  OBJECT_BALL,             /*!< Ball                     00 */
  OBJECT_GOAL_L_T,         /*!< Left top goal post       01 */   // 4 goals
  OBJECT_GOAL_L_B,         /*!< Left bottom goal post    02 */
  OBJECT_GOAL_R_T,         /*!< Right top goal post      03 */
  OBJECT_GOAL_R_B,         /*!< Right bottom goal post   04*/
  OBJECT_GOAL_UNKNOWN,     /*!< Unknown goal             05 */
  OBJECT_CORNER_L_T,       /*!< Flag corner left top     06 */   // 4 flags
  OBJECT_CORNER_L_B,       /*!< Flag corner left bottom  07 */
  OBJECT_CORNER_R_T,       /*!< Flag corner right top    08 */
  OBJECT_CORNER_R_B,       /*!< Flag corner right bottom 09 */
  OBJECT_CORNER_UNKNOWN,   /*!< Unknown corner           10 */
  OBJECT_TEAMMATE_1,       /*!< Teammate nr 1            11 */   // teammates 12
  OBJECT_TEAMMATE_2,       /*!< Teammate nr 2            12 */
  OBJECT_TEAMMATE_3,       /*!< Teammate nr 3            13 */
  OBJECT_TEAMMATE_4,       /*!< Teammate nr 4            14 */
  OBJECT_TEAMMATE_5,       /*!< Teammate nr 5            15 */
  OBJECT_TEAMMATE_6,       /*!< Teammate nr 6            16 */
  OBJECT_TEAMMATE_7,       /*!< Teammate nr 7            17 */
  OBJECT_TEAMMATE_8,       /*!< Teammate nr 8            18 */
  OBJECT_TEAMMATE_9,       /*!< Teammate nr 9            19 */
  OBJECT_TEAMMATE_10,      /*!< Teammate nr 10           20 */
  OBJECT_TEAMMATE_11,      /*!< Teammate nr 11           21 */
  OBJECT_TEAMMATE_UNKNOWN, /*!< Teammate nr unkown       22 */
  OBJECT_OPPONENT_1,       /*!< Opponent nr 1            23 */   // opponents 24
  OBJECT_OPPONENT_2,       /*!< Opponent nr 2            24 */
  OBJECT_OPPONENT_3,       /*!< Opponent nr 3            25 */
  OBJECT_OPPONENT_4,       /*!< Opponent nr 4            26 */
  OBJECT_OPPONENT_5,       /*!< Opponent nr 5            27 */
  OBJECT_OPPONENT_6,       /*!< Opponent nr 6            28 */
  OBJECT_OPPONENT_7,       /*!< Opponent nr 7            29 */
  OBJECT_OPPONENT_8,       /*!< Opponent nr 8            30 */
  OBJECT_OPPONENT_9,       /*!< Opponent nr 9            31 */
  OBJECT_OPPONENT_10,      /*!< Opponent nr 10           32 */
  OBJECT_OPPONENT_11,      /*!< Opponent nr 11           33 */
  OBJECT_OPPONENT_UNKNOWN, /*!< Opponent nr unknown      34 */
  OBJECT_PLAYER_UNKNOWN,   /*!< Unknown player           35 */
  OBJECT_TEAMMATE_GOALIE,  /*!< Goalie of your side      36 */
  OBJECT_OPPONENT_GOALIE,  /*!< Goalie of opponent side  37 */
  MAX_OBJECTS,             /*!< maximum nr of objects    39 */
  OBJECT_ILLEGAL,          /*!< illegal object           38 */
} ;

#define hipXYL JID_LLEG_1        /*!< Human understandable joint name :) */
#define hipXYR JID_RLEG_1        /*!< Human understandable joint name :) */

#define hipL JID_LLEG_2_3        /*!< Human understandable joint name :) */
#define hipR JID_RLEG_2_3        /*!< Human understandable joint name :) */

#define kneeL JID_LLEG_4         /*!< Human understandable joint name :) */
#define kneeR JID_RLEG_4         /*!< Human understandable joint name :) */

#define ankleL JID_LLEG_5_6      /*!< Human understandable joint name :) */
#define ankleR JID_RLEG_5_6      /*!< Human understandable joint name :) */

#define shoulderL JID_LARM_1_2   /*!< Human understandable joint name :) */
#define shoulderR JID_RARM_1_2   /*!< Human understandable joint name :) */

#define shoulderXYL JID_LARM_3   /*!< Human understandable joint name :) */
#define shoulderXYR JID_RARM_3   /*!< Human understandable joint name :) */

#define elbowL JID_LARM_4        /*!< Human understandable joint name :) */
#define elbowR JID_RARM_4        /*!< Human understandable joint name :) */


/*! This is an enumeration of all the joints currently avalable in simulator */
enum JointT
{
  JID_LLEG_1,                  /*!< Left hip joint - moving on XY plane */
  JID_RLEG_1,                  /*!< Right hip joint - moving on XY plane */
  JID_LLEG_2_3,                /*!< Left hip joint - moving front & back, open & close */
  JID_RLEG_2_3,                /*!< Right hip joint - moving front & back, open & close */
  JID_LLEG_4,                  /*!< Left knee joint */
  JID_RLEG_4,                  /*!< Right knee joint */
  JID_LLEG_5_6,                /*!< Left ankle joint */
  JID_RLEG_5_6,                /*!< Right ankle joint */
  JID_LARM_1_2,                /*!< Left shoulder joint - moving front & back, open & close */
  JID_RARM_1_2,                /*!< Right shoulder joint - moving front & back, open & close */
  JID_LARM_3,                  /*!< Left shoulder joint - moving on XY plane */
  JID_RARM_3,                  /*!< Right shoulder joint - moving on XY plane */
  JID_LARM_4,                  /*!< Left elbow joint */
  JID_RARM_4,                  /*!< Right elbow joint */
  MAX_JOINTS,                  /*!< Maximum number of joints */
  JID_ILLEGAL                  /*!< unknown joint :( ! */
} ;

/*! This is an enumeration of the sensros available currently in the simulator */
enum SensorT
{
  SENSE_FOOT_LEFT,             /*!< Sensor under left foot */
  SENSE_FOOT_RIGHT,            /*!< Sensor under right foot */
  MAX_SENSORS,                 /*!< Maximum number of sensors */
  SENSE_ILLEGAL                /*!< Unknown sensor */
};

/*!The ObjectSetT enumerations holds the different object sets, which
   consists of one or multiple ObjectT types. */
enum ObjectSetT
{
  OBJECT_SET_TEAMMATES,        /*!< teammates                       */
  OBJECT_SET_OPPONENTS,        /*!< opponents                       */
  OBJECT_SET_PLAYERS,          /*!< players                         */
  OBJECT_SET_TEAMMATES_NO_GOALIE,/*!< teammates without the goalie  */
  OBJECT_SET_FLAGS,            /*!< flags                           */
  OBJECT_SET_ILLEGAL           /*!< illegal                         */
} ;

/*!The PlayModeT enumeration contains all playmodes of the soccer playmode.
   The associated string values can be returned using the methods in the
   SoccerTypes class */
enum PlayModeT {
  PM_BEFORE_KICK_OFF = 0,    /*!< before_kick_off:   before kick off         */
  PM_KICK_OFF_LEFT,          /*!< kick_off_l:        kick off for left team  */
  PM_KICK_OFF_RIGHT,         /*!< kick_off_r:        kick off for right team */
  PM_PLAY_ON,                /*!< play_on:           play on (during match)  */
  PM_KICK_IN_LEFT,           /*!< kick_in_l:         kick in for left team   */
  PM_KICK_IN_RIGHT,          /*!< kick_in_r:         kick in for right team  */
  PM_CORNER_KICK_LEFT,       /*!< corner_kick_l:     corner kick left team   */
  PM_CORNER_KICK_RIGHT,      /*!< corner_kick_r:     corner kick right team  */
  PM_GOAL_KICK_LEFT,         /*!< goal_kick_l:       goal kick for left team */
  PM_GOAL_KICK_RIGHT,        /*!< goal_kick_r:       goal kick for right team*/
  PM_OFFSIDE_LEFT,           /*!< offside_l:         offside for left team   */
  PM_OFFSIDE_RIGHT,          /*!< offside_r:         offside for right team  */
  PM_QUIT,                   /*!< quit                                       */
  PM_GOAL_LEFT,              /*!< goal_l:            goal scored by team left*/
  PM_GOAL_RIGHT,             /*!< goal_r:            goal scored by team righ*/
  PM_FREE_KICK_LEFT,         /*!< free_kick_l:       free kick for left team */
  PM_FREE_KICK_RIGHT,        /*!< free_kick_r:       free kick for right team*/
  PM_ILLEGAL                 /*!< unknown playmode                           */
} ;

/*!The SideT enumeration contains the two sides */
enum SideT {
  SIDE_NONE = 0, /*!< Our Side     */
  SIDE_LEFT,     /*!< Left side    */
  SIDE_RIGHT,    /*!< Right side   */
  SIDE_ILLEGAL   /*!< Illegal side */
} ;

/*!The CommandT enumeration contains the different types for the SoccerCommand
   that are possible. */
enum CommandT {
  CMD_ILLEGAL,      /*!< illegal command             */
  CMD_JOINT,        /*!< Joint handling command      */
  CMD_SAY,          /*!< say command                 */
  CMD_MAX_COMMANDS  /*!< maximum number of commands  */
} ;

/*!The PlayerT enumeration contains the different playertypes that are defined
   in a formation. This should not be confused with the later introducted
   player_types in soccerserver 7.xx that denotes the different possible
   heterogeneous players. A player type in the context PlayerT denotes the
   kind of player (midfielder, attacker) on the field. Its role on the pitch.*/
enum PlayerT {
  PT_ILLEGAL,             /*!< illegal player type */
  PT_GOALKEEPER,          /*!< goalkeeper          */
  PT_DEFENDER_CENTRAL,    /*!< central defender    */
  PT_DEFENDER_SWEEPER,    /*!< sweeper defender    */
  PT_DEFENDER_WING,       /*!< wing defender       */
  PT_MIDFIELDER_CENTER,   /*!< central midfielder  */
  PT_MIDFIELDER_WING,     /*!< wing midfielder     */
  PT_ATTACKER_WING,       /*!< wing attacker       */
  PT_ATTACKER,            /*!< central attacker    */
  MAX_PLAYER_TYPES
} ;

/*! The PlayerSetT enumeration contains different sets of playertypes that
    are defined in a formation. Possible sets are all midfielders, etc. */
enum PlayerSetT {
  PS_DEFENDERS,           /*!< all defenders       */
  PS_MIDFIELDERS,         /*!< all midfielders     */
  PS_ATTACKERS,           /*!< all attackers       */
  PS_ALL                  /*!< all players         */
} ;

/*!The FormationT enumeration contains the different formation types that
   are defined. */
enum FormationT {
  FT_ILLEGAL,             /*!< illegal formation type                  */
  FT_INITIAL,             /*!< initial formation type (before kick off)*/
  FT_433_OFFENSIVE,       /*!< 433 offensive formation                 */
  FT_334_OFFENSIVE,       /*!< 434 offensive formation                 */
  FT_343_ATTACKING,       /*!< attacking formation type                */
  MAX_FORMATION_TYPES
} ;

/*! The MarkT enumeration contains different marking techniques. */
enum MarkT {
  MARK_ILLEGAL,    /*!< illegal marking                                     */
  MARK_GOAL,       /*!< mark goal (stand on obj-goal line)                  */
  MARK_BISECTOR,   /*!< mark bisector stand between goal,obj and ball,obj   */
  MARK_BALL        /*!< mark ball (stand on obj-ball line)                  */
} ;

/*! The ClearBallT enumeration contains different clear ball possibilities. */
enum ClearBallT {
  CLEAR_BALL_ILLEGAL,    /*!< illegal clear ball                            */
  CLEAR_BALL_OFFENSIVE,  /*!< clear ball to the front of the field          */
  CLEAR_BALL_DEFENSIVE,  /*!< clear ball to the center line of the field    */
  CLEAR_BALL_OFFENSIVE_SIDE, /*!< clear ball to front and side of the field */
  CLEAR_BALL_GOAL        /*!< clear ball to position in front of the goal   */
} ;

/*! The SucceedT enumeration contains the different succeed rate probabilites*/
enum SucceedT  {
  SUCCEED_ILLEGAL,             /*!< illegal message                          */
  SUCCEED_ALWAYS,              /*!< wil always succeed                       */
  SUCCEED_DOUBTFUL,            /*!< in some occassions it may succeed        */
  SUCCEED_NEVER                /*!< this will never succeed                  */
} ;


/*****************************************************************************/
/********************* CLASS SOCCERCOMMAND ***********************************/
/*****************************************************************************/

/*!This class resembles a SoccerCommand that contains all the information about
   a command that can be sent to the server, but the format is independent from
   the server implementation. A SoccerCommand can be created and before it is
   sent to the server, be converted to the actual string representation
   understood by the server. */
class SoccerCommand
{
  // private methods to generate text string to send to server
  bool  makeCommandString       ( char *str                         ) const;

public:

  // different variables that are used by the different possible commands
  // only the variables that are related to the current commandType have
  // legal values
  CommandT     commandType;      /*!< type of this command                */
  JointT       JID;              /*!< Joint to change within this command */
  double       dSpeed1;          /*!< speed of first angle of joint       */
  double       dSpeed2;          /*!< speed of second angle joint         */
  char         str[MAX_SAY_MSG]; /*!< str (for say)                       */

  SoccerCommand                 ( CommandT com = CMD_ILLEGAL,
                                             JointT jid = JID_ILLEGAL,
                                             double d1=UnknownDoubleValue,
                                             double d2=UnknownDoubleValue );
  SoccerCommand                 ( CommandT com, char *msg                 );

  // command to set the different values of the SoccerCommand
  void makeCommand              ( CommandT com, JointT jid,
                                  double     d1 = UnknownDoubleValue,
                                  double     d2 = UnknownDoubleValue      );
  void makeCommand              ( CommandT com, char *msg                 );

  bool isIllegal                (                                         );

  friend ostream& operator <<   (ostream &os, SoccerCommand cmd           );
  void show                     ( ostream& os = cout                      );

  // used to return the string representation of this SoccerCommand
  bool getCommandString         ( char *str                               ) const;
} ;

/*****************************************************************************/
/********************** CLASS SOCCERTYPES ************************************/
/*****************************************************************************/

/*! The class SoccerTypes contains different methods to work with the
    different enumerations defined in SoccerTypes.h. It is possible to
    convert soccertypes to strings and strings to soccertypes. It is
    also possible to get more specific information about some of the
    soccertypes. All methods are static so it is possible to call the
    methods without instantiating the class. */
class SoccerTypes
{
 public:
  // methods that deal with the different objects
  static char*           getObjectStr              ( char         *strBuf,
                                                     ObjectT      o          );
  static ObjectT         getObjectFromStr          ( char         *str       );
  static bool            isInSet                   ( ObjectT      o,
                                                     ObjectSetT   o_g,
                                                     ObjectT      objectGoalie=
                                                       OBJECT_TEAMMATE_1     );
  static bool            isPlayerTypeInSet         ( PlayerT      p,
						     PlayerSetT   p_s        );
  static bool            isGoal                    ( ObjectT      o          );
  static bool            isFlag                    ( ObjectT      o          );
  static bool            isBall                    ( ObjectT      o          );
  static bool            isTeammate                ( ObjectT      o          );
  static bool            isOpponent                ( ObjectT      o          );
  static bool            isGoalie                  ( ObjectT      o          );
  static bool            isPlayer                  ( ObjectT      o          );
  static int             getIndex                  ( ObjectT      o          );
  static ObjectT         getTeammateObjectFromIndex( int          iIndex     );
  static ObjectT         getOpponentObjectFromIndex( int          iIndex     );
  static bool            isKnownPlayer             ( ObjectT o               );

  // methods that deal with the different play modes
  static const char*     getPlayModeStr            ( PlayModeT    p          );
  static PlayModeT       getPlayModeFormStr        ( const char *str         );

  // methods that deal with the side information
  static const char*     getSideStr                ( SideT        s          );
  static SideT           getSideFromStr            ( char*        str        );
  
  // methods that deal with joint namings
  static JointT          getJointFromStr           ( const char * str        );
  static const char *    getJointStr               ( JointT joint            );
  static const char *    getJointServerMsg         ( JointT joint            );

  // methods that deal with sensors
  static const char *    getSensorString           ( SensorT sense           );
  static const SensorT   getSensorFromString       ( const char *str         );
} ;

#endif
